@@ -15,7 +15,15 @@ package Safe::World::stderr ;
 use strict qw(vars);
 
 our ($VERSION , @ISA) ;
-$VERSION = '0.01' ;
+$VERSION = '0.02' ;
+
+##########
+# SCOPES #
+##########
+
+  use vars qw($Safe_World_NOW) ;
+  
+  *Safe_World_NOW = \$Safe::World::NOW ;
 
 #########
 # PRINT #
@@ -35,15 +43,15 @@ sub print_stderr {
   
   if ( ref($stderr) eq 'SCALAR' ) { $$stderr .= $_[0] ;}
   elsif ( ref($stderr) eq 'CODE' ) {
-    my $sel = select( $Safe::World::NOW->{SELECT}{PREVSTDOUT} ) if $Safe::World::NOW->{SELECT}{PREVSTDOUT} ;
-    &$stderr($Safe::World::NOW , $_[0]) ;
+    my $sel = select( $Safe_World_NOW->{SELECT}{PREVSTDOUT} ) if $Safe_World_NOW->{SELECT}{PREVSTDOUT} ;
+    &$stderr($Safe_World_NOW , $_[0]) ;
     select($sel) if $sel ;
   }
   else {
     my $sel = \*main::STDERR ;
-    *main::STDERR = $Safe::World::NOW->{SELECT}{PREVSTDERR} if $Safe::World::NOW->{SELECT}{PREVSTDERR} ;
+    *main::STDERR = $Safe_World_NOW->{SELECT}{PREVSTDERR} if $Safe_World_NOW->{SELECT}{PREVSTDERR} ;
     print $stderr $_[0] ;
-    *main::STDERR = $sel if $Safe::World::NOW->{SELECT}{PREVSTDERR} ;
+    *main::STDERR = $sel if $Safe_World_NOW->{SELECT}{PREVSTDERR} ;
   }
 
   return 1 ;
